#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using System;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientOperationsLib;
using VaultLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This file checks a file in to the repository.
	/// </summary>
	/// <example>
	/// <code>
	/// <![CDATA[
	///     <vaultcommit
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository"
	///			repositorypath="\Project\Directory\File.cs" />
	///			comment="comment"
	///			batchcommit="true"
	///			failonerror="false" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultcommit")]
	public class VaultCommit : VaultBase
	{
		#region Private Variable Declarations.
		private string	pRepository		= "";
		private	string	pComment		= "";
		private string	pRepositoryPath	= "";
		private bool	bBatchCommit	= true;
		#endregion
		
		#region Accessor Declarations.
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		/// <summary>
		/// The path to commit from the specified repository.
		/// </summary>
		[TaskAttribute( "repositorypath", Required = true )]
		public string RepositoryPath
		{
			get { return pRepositoryPath;	}
			set { pRepositoryPath = value;	}
		}

		/// <summary>
		/// Comment to attach to the files and/or folders being checked out.
		/// </summary>
		[TaskAttribute( "comment", Required = false )]
		public string Comment
		{
			get	{ return pComment;	}
			set	{ pComment = value;	}
		}

		/// <summary>
		/// Gets or sets a value indicating whether [batch commit].
		/// </summary>
		/// <value>
		/// 	<c>true</c> if [batch commit]; otherwise, <c>false</c>.
		/// </value>
		[TaskAttribute( "batchcommit", Required = false )]
		public bool BatchCommit
		{
			get	{ return bBatchCommit;	}
			set	{ bBatchCommit = value;	}
		}
		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		/// <exception>BuildException</exception>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try 
			{
				SelectRepository( Repository );
				Commit( RepositoryPath );			
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// Commit changes in the current change set.
		/// </summary>
		/// <exception>BuildException</exception>
		protected void Commit( string strPath )
		{
			if(IsVaultFolder( strPath ))
			{
				strPath = AppendLastSlash(strPath);
			}

			ClientInstance.DontUseSendChunked = false;
			ClientInstance.WorkingFolderOptions.RequireCheckOutBeforeCheckIn = false;
			ClientInstance.UpdateKnownChanges_All(false);

			ChangeSetItemColl csic = ClientInstance.InternalChangeSet_GetItems(true);
			
			//remove change set items that do not correspond to this project

			// Reverse through the array, so that when an item is removed, 
			// it affects the area already worked on, thus keeping things simple. 
			for( int i = csic.Count - 1; i >= 0; i-- ) 
			{ 
				ChangeSetItem csi = csic[i]; 

				string strDisplayRepositoryPath = csi.DisplayRepositoryPath;
				string strRepositoryPath = null;
				int nIndex = strDisplayRepositoryPath.LastIndexOf('$');
				if (nIndex >= 0)
				{
					if (strDisplayRepositoryPath.Length >= strPath.Length)
					{
						strRepositoryPath = strDisplayRepositoryPath.Substring(0, strPath.Length);
					}
				}
				
				if (strRepositoryPath == strPath)
				{
					//set the comment for the change set item
					((ChangeSetItem)csic[i]).Comment = Comment;

					//resolve merge status
					switch (((ChangeSetItem)csic[i]).Type)
					{
						case ChangeSetItemType.Modified:
							if (((ChangeSetItem_Modified)csic[i]).NeedsMerge)
							{
								VaultResolveMergeStatus resolve = new VaultResolveMergeStatus();
								resolve.Url = this.Url;
								resolve.Username = this.Username;
								resolve.Password = this.Password;
								resolve.Repository = this.Repository;
								resolve.CheckOutFileName = ((ChangeSetItem)csic[i]).DisplayRepositoryPath;

								Log(Level.Info, "Resolving merge status: " + resolve.CheckOutFileName);

								resolve.Execute();
							}
							break;
					}
					
				}
				else
				{
					csic.RemoveAt(i);
				}
			}

			if (BatchCommit)
			{
				if (csic.Count > 0)
				{
					if( !ClientInstance.Commit( csic, true, false) )
					{
						throw new Exception("Failed to commit " + csic.Count.ToString() + " change set items.");
					}
				}
			}
			else
			{
				//commit one change set item at a time so that we can keep track of change set items that failed to commit
				ChangeSetItemColl faileditems = new ChangeSetItemColl();

				if (csic.Count > 0)
				{
					ChangeSetItemColl csic2 = new ChangeSetItemColl();

					foreach (ChangeSetItem csi2 in csic)
					{
						csic2.Clear();
						csic2.Add(csi2);

						try
						{
							if( !ClientInstance.Commit( csic2, true, false) )
							{
								faileditems.Add(csi2);
							}
						}
						catch (System.Exception e)
						{
							VaultLog("Failed to commit " + csi2.DisplayRepositoryPath + " change set item." + e.ToString());
							faileditems.Add(csi2);
						}
						
					}
				}

				if (faileditems.Count > 0)
				{
					foreach (ChangeSetItem faileditem in faileditems)
					{
						Log(Level.Info, faileditem.DisplayName + " could not be committed." );
					}
				}
			}

			VaultLog( "The changeset for " + strPath + " was successfully committed." );
		}

		private string AppendLastSlash(string strPath)
		{
			if (strPath.Length > 0)
			{
				if (strPath[strPath.Length - 1] != '/')
				{
					return strPath + "/";
				}
			}

			return strPath;
		}

		#endregion
	}
}
