#region GPL License.

/* 

 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
#endregion

using System;
using System.Collections;
using System.Text.RegularExpressions;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;
using VaultClientNetLib;
using VaultClientOperationsLib;
using VaultLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task is used to add new file(s) to a Vault repository.
	/// </summary>
	/// <example>
	/// <code>
	/// <![CDATA[
	///     <vaultfindfile
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository"
	///			path="\Project\Directory\"
	///			filename="file1.txt"
	///			exactmatch="false"
	///			casesensitive="true"
	///			foundfileproperty="propertyname"
	///			 />
	///   ]]></code>
	/// </example>
	[TaskName("vaultfindfile")]
	public class VaultFindFile : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		private	string				pDirectoryPath			= "";
		private	string				pFileName				= "";
		private	string				pComment				= "";
		private bool pExactMatch = false;
		private bool pCaseSensitive = true;
		private string pFoundFileProperty = "";
		#endregion

		#region Attributes
		/// <summary>
		/// The name of the Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		/// <summary>
		/// Comment to attach to the files and/or folders being added.
		/// </summary>
		[TaskAttribute( "comment", Required = false )]
		public string Comment
		{
			get	{ return pComment;							}
			set	{ pComment = value;							}
		}

		/// <summary>
		/// Root path in the repository to store the data.
		/// </summary>
		[TaskAttribute( "path", Required = true )]
		public string DirectoryPath
		{
			get	{ return pDirectoryPath;					}
			set	{ pDirectoryPath = value;					}
		}

		/// <summary>
		/// Root path in the repository to store the data.
		/// </summary>
		[TaskAttribute( "filename", Required = true )]
		public string FileName
		{
			get	{ return pFileName;						}
			set	{ pFileName = value;					}
		}

		/// <summary>
		/// Root path in the repository to store the data.
		/// </summary>
		[TaskAttribute( "exactmatch")]
		public bool ExactMatch
		{
			get	{ return pExactMatch;						}
			set	{ pExactMatch = value;					}
		}

		/// <summary>
		/// Root path in the repository to store the data.
		/// </summary>
		[TaskAttribute( "casesensitive")]
		public bool CaseSensitive
		{
			get	{ return pCaseSensitive;						}
			set	{ pCaseSensitive = value;					}
		}

		/// <summary>
		/// Root path in the repository to store the data.
		/// </summary>
		[TaskAttribute( "foundfileproperty", Required = true)]
		public string FoundFileProperty
		{
			get	{ return pFoundFileProperty;						}
			set	{ pFoundFileProperty = value;					}
		}
		#endregion
	
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		/// <exception>BuildException</exception>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try 
			{
				SelectRepository( Repository );
				
				ArrayList returnArray = new ArrayList();

				DirectoryPath = RepositoryPath.NormalizeFolder( DirectoryPath );

				VaultClientFolder vaultFolder = ClientInstance.TreeCache.Repository.Root.FindFolderRecursive(DirectoryPath);

				if( vaultFolder == null )
				{
					throw new BuildException( "Error validating path \"" + DirectoryPath + "\".", Location );
				}

				SearchForFileName(vaultFolder, FileName, returnArray, ExactMatch, CaseSensitive);

				if (returnArray.Count == 0)
				{
					throw new BuildException( "File not found \"" + FileName + "\".", Location );
				}
				
				Project.Properties[FoundFileProperty] = ((VaultClientFile)returnArray[0]).FullPath;
				
			}
			finally
			{
				Logout();
			}
		}
		#endregion

		#region Support functions.

		private void SearchForFileName(
				VaultClientFolder vaultFolder,
				string fileName,
				ArrayList returnArray,
				bool exactMatch,
				bool caseSensitive)
		{
			if (vaultFolder.Files != null)
			{
				foreach (VaultClientFile subfile in vaultFolder.Files)
				{
					if (exactMatch)
					{
						if ((caseSensitive == false) && (subfile.Name.ToUpper() == fileName.ToUpper()))
						{
							returnArray.Add(subfile);
						}
						else if (subfile.Name == fileName)
						{
							returnArray.Add(subfile);
						}
					}
					else
					{
						if ((caseSensitive == false) && Regex.IsMatch(subfile.Name, fileName, RegexOptions.IgnoreCase))
						{
							returnArray.Add(subfile);
						}
						else if (Regex.IsMatch(subfile.Name, fileName))
						{
							returnArray.Add(subfile);
						}
					}
				}
			}

			if (vaultFolder.Folders != null)
			{
				foreach (VaultClientFolder subfolder in vaultFolder.Folders)
				{
					SearchForFileName(subfolder, fileName, returnArray, exactMatch, caseSensitive);
				}
			}
		}
		#endregion
	}
}

