#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using System;
using System.Collections;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientOperationsLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task lists all of the files in a specified folder and requires appropriate access
	/// permissions.
	/// </summary>
	/// <example>
	///   <code><![CDATA[
	///     <vaultlistchangeset
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository"
	///			path="/Project/Directory" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultlistchangeset")]
	public class VaultListChangeSet : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		private	string				pDirectoryPath			= "";
		#endregion
		
		#region Accessor Declarations
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		/// <summary>
		/// Gets or sets the directory path.
		/// </summary>
		/// <value></value>
		[TaskAttribute( "path", Required = true )]
		public string DirectoryPath
		{
			get	{ return pDirectoryPath;					}
			set	{ pDirectoryPath = value;					}
		}
		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try
			{
				SelectRepository( Repository );
				ListChangeSet();
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// This function will list the current change set in the current client instance.
		/// </summary>
		protected void ListChangeSet()
		{
			ClientInstance.UpdateKnownChanges_All(true);

			ChangeSetItemColl csic = ClientInstance.InternalChangeSet_GetItems(true);

			WriteChangeSet(csic);
		}

		/// <summary>
		/// This small function simply formats and displays the data.
		/// </summary>
		/// <param name="fileName">Filename to display.</param>
		/// <param name="fileLength">Length of file to display.</param>
		/// <param name="fileVersion">Version of file to display.</param>
		protected void LogDetails( string fileName, string fileLength, string fileVersion )
		{
			VaultLog( fileName.PadRight( 16, ' ' ) + fileLength.PadRight( 16, ' ' ) + fileVersion );
		}

		/// <summary>
		/// Writes the change set.
		/// </summary>
		/// <param name="csic">Csic.</param>
		public void WriteChangeSet(ChangeSetItemColl csic)
		{
			Stream stream = Console.OpenStandardOutput();
			StreamWriter sw = new StreamWriter(stream);
			XMLOutputWriter _xml = new XMLOutputWriter(sw);

			if ( _xml != null )
			{
				_xml.Begin("changeset");
			}

			try
			{
				if ( (csic != null) && (csic.Count > 0) )
				{
					int i, nCnt;
					ChangeSetItem csi = null;
					for (i = 0, nCnt = csic.Count; i<nCnt; i++)
					{
						csi = csic[i];
						switch (csi.Type)
						{
							case ChangeSetItemType.AddFile:
							{
								ChangeSetItem_AddFile it = (ChangeSetItem_AddFile) csi;
							
								_xml.Begin("AddFile");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.AddPair("localpath", it.DiskFile);
								_xml.End();
							
								break;
							}
							case ChangeSetItemType.AddFolder:
							{
								ChangeSetItem_AddFolder it = (ChangeSetItem_AddFolder) csi;

								_xml.Begin("AddFolder");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("reposfolder", it.DisplayRepositoryPath);
								_xml.AddPair("localfolder", it.DiskFolder);
								_xml.End();

								break;
							}
							case ChangeSetItemType.DeleteFile:
							{
								ChangeSetItem_DeleteFile it = (ChangeSetItem_DeleteFile) csi;

								_xml.Begin("DeleteFile");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.DeleteFolder:
							{
								ChangeSetItem_DeleteFolder it = (ChangeSetItem_DeleteFolder) csi;

								_xml.Begin("DeleteFolder");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.CreateFolder:
							{
								ChangeSetItem_CreateFolder it = (ChangeSetItem_CreateFolder) csi;

								_xml.Begin("CreateFolder");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Branch:
							{
								ChangeSetItem_CopyBranch it = (ChangeSetItem_CopyBranch) csi;

								_xml.Begin("Branch");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.AddPair("branchpath", it.BranchPath);
								_xml.End();
							
								break;
							}
							case ChangeSetItemType.Share:
							{
								ChangeSetItem_Share it = (ChangeSetItem_Share) csi;

								_xml.Begin("Share");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.RepositoryPath);
								_xml.AddPair("sharepath", it.NewSharePath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Pin:
							{
								ChangeSetItem_Pin it = (ChangeSetItem_Pin) csi;

								_xml.Begin("Pin");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Rename:
							{
								ChangeSetItem_Rename it = (ChangeSetItem_Rename) csi;

								_xml.Begin("Rename");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.OldRepositoryPath);
								_xml.AddPair("newname", it.NewName);
								_xml.End();
								break;
							}
							case ChangeSetItemType.Unpin:
							{
								ChangeSetItem_Unpin it = (ChangeSetItem_Unpin) csi;

								_xml.Begin("Unpin");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Move:
							{
								ChangeSetItem_Move it = (ChangeSetItem_Move) csi;

								_xml.Begin("Move");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.OldRepositoryPath);
								_xml.AddPair("newpath", it.NewOwnerPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Modified:
							{
								ChangeSetItem_Modified it = (ChangeSetItem_Modified) csi;

								_xml.Begin("ModifyFile");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.AddPair("localpath", it.DiskFile);
								_xml.AddPair("needsmerge", it.NeedsMerge);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Unmodified:
							{
								ChangeSetItem_Unmodified it = (ChangeSetItem_Unmodified) csi;

								_xml.Begin("UnmodifiedFile");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("respospath", it.DisplayRepositoryPath);
								_xml.AddPair("localpath", it.DiskFile);
								_xml.End();

								break;
							}
							case ChangeSetItemType.ChangeExtProperties:
							{
								ChangeSetItem_ChangeExtProperties it = (ChangeSetItem_ChangeExtProperties) csi;

								_xml.Begin("ChangeExtProperties");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("repospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.CheckedOutMissing:
							{
								ChangeSetItem_CheckedOutMissing it = (ChangeSetItem_CheckedOutMissing) csi;

								_xml.Begin("CheckedOutMissing");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("respospath", it.DisplayRepositoryPath);
								_xml.End();

								break;
							}
							case ChangeSetItemType.Snapshot:
							{
								ChangeSetItem_Snapshot it = (ChangeSetItem_Snapshot)csi;
								_xml.Begin("Snapshot");
								_xml.AddPair("id", i.ToString());
								_xml.AddPair("respospath", it.RepositoryPath);
								_xml.AddPair("parentpath", it.SnapshotPath);
								_xml.End();

								break;
							}
							default:
							{
								// this should never happen.
								throw new Exception("ACK!  There is a ChangeSet item we don't recognize.");
							}
						}
					}
				}
			}
			finally
			{
				// end the xml pair.
				if ( _xml != null )
				{
					_xml.End();
				}
			}

			sw.Flush();
			sw.Close();
		}
		#endregion
	}
}
