#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientNetLib.ClientService;
using VaultClientOperationsLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task lists all of the shared files and folders in a specified folder
	/// permissions.
	/// </summary>
	/// <example>
	///   <code><![CDATA[
	///     <vaultlistshares url="http://localhost" username="username" password="password" path="/Project/Directory" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultlistshares")]
	public class VaultListShares : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		private	string				pRepositoryPath			= "";
		private	string				pExcludePath			= "";
		#endregion
		
		#region Accessor Declarations
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		/// <summary>
		/// Gets or sets the repository path.
		/// </summary>
		/// <value></value>
		[TaskAttribute( "repositorypath", Required = true )]
		public string RepositoryPath
		{
			get	{ return pRepositoryPath;					}
			set	{ pRepositoryPath = value;					}
		}

		/// <summary>
		/// Gets or sets the exclude path.
		/// </summary>
		/// <value>The exclude path.</value>
		[TaskAttribute( "excludepath", Required = false )]
		public string ExcludePath
		{
			get	{ return pExcludePath;					}
			set	{ pExcludePath = value;					}
		}
		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try
			{
				SelectRepository( Repository );

				if( IsVaultFolder( RepositoryPath ) )
				{
					ListSharesInDirectory(RepositoryPath);
				}
				else if (IsVaultFile(RepositoryPath))
				{
					ListFileShares(RepositoryPath);
				}
				else
				{
					throw new BuildException( RepositoryPath + " not found.  Listing aborted.", Location );
				}
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// Given a path this function will enumerate the files contained within that path on the Vault server.
		/// </summary>
		/// <param name="directoryPath">Path to list files from.</param>
		protected void ListSharesInDirectory( string directoryPath )
		{
			VaultClientFolder vaultClientFolder = ClientInstance.TreeCache.Repository.Root.FindFolderRecursive( directoryPath );

			VaultFolderProperties folderProperties = ClientInstance.GetFolderProperties(vaultClientFolder.FullPath, vaultClientFolder.ObjVerID);

			string[] sharedPaths = folderProperties.SharedPaths;

			if (OutputShares(sharedPaths) == 1)
			{
				//folder is not shared
				
				//so act recursively on sub files and folders

				foreach( VaultClientFolder vaultClientSubFolder in vaultClientFolder.Folders )
				{
					ListSharesInDirectory( vaultClientSubFolder.FullPath );
				}

				foreach( VaultClientFile vaultClientFile in vaultClientFolder.Files )
				{
					ListFileShares(vaultClientFile);
				}
			}
		}

		/// <summary>
		/// Lists the file shares.
		/// </summary>
		/// <param name="filePath">The file path.</param>
		protected void ListFileShares( string filePath )
		{
			VaultClientFile vaultClientFile = ClientInstance.TreeCache.Repository.Root.FindFileRecursive( filePath );

			ListFileShares(vaultClientFile);
		}

		/// <summary>
		/// Lists the file shares.
		/// </summary>
		/// <param name="vaultClientFile">The vault client file.</param>
		protected void ListFileShares( VaultClientFile vaultClientFile )
		{
			VaultFileProperties fileProperties = ClientInstance.GetFileProperties(vaultClientFile.FullPath, vaultClientFile.ObjVerID);

			OutputShares(fileProperties.SharedPaths);
		}

		/// <summary>
		/// This function will output a given array of strings
		/// </summary>
		/// <param name="shares">Path to list files from.</param>
		protected int OutputShares( string[] shares )
		{
			int nLength = shares.Length;

			Project.Log(Level.Info, "Number of shares: " + nLength.ToString());

			if (nLength > 1)
			{
				foreach (string strShare in shares)
				{
					Project.Log(Level.Info, strShare);

					if (!(strShare.IndexOf(ExcludePath) >= 0))
					{
						Project.Log(Level.Info, strShare);
					}
				}
			}

			return nLength;
		}
		#endregion
	}
}
