#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using System;
using System.Collections;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientOperationsLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task lists all of the files in a specified folder and requires appropriate access
	/// permissions.
	/// </summary>
	/// <example>
	///   <code><![CDATA[
	///     <vaultlistchangeset
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultlistworkingfolders")]
	public class VaultListWorkingFolders : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		#endregion
		
		#region Accessor Declarations
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}
		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try
			{
				SelectRepository( Repository );
				WriteListWorkingFolders();
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// Writes the list working folders.
		/// </summary>
		protected void WriteListWorkingFolders()
		{
			string[] repos = null;
			string[] disk = null;

			ClientInstance.TreeCache.GetWorkingFolderAssignments(ref repos, ref disk);

			Stream stream = Console.OpenStandardOutput();
			StreamWriter sw = new StreamWriter(stream);
			XMLOutputWriter _xml = new XMLOutputWriter(sw);

			_xml.Begin("listworkingfolders");
			for (int i=0; i<repos.Length; i++)
			{
				_xml.Begin("workingfolder");
				_xml.AddPair("reposfolder", repos[i]);
				_xml.AddPair("localfolder", disk[i]);
				_xml.End();
			}
			_xml.End();

			sw.Flush();
			sw.Close();
		}
		#endregion
	}
}
