#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using System;
using System.Collections;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientOperationsLib;
using VaultLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task lists all of the files in a specified folder and requires appropriate access
	/// permissions.
	/// </summary>
	/// <example>
	///   <code><![CDATA[
	///     <vaultrefreshchangeset
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultrefreshchangeset")]
	public class VaultRefreshChangeSet : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		#endregion
		
		#region Accessor Declarations
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try
			{
				SelectRepository( Repository );
				RefreshChangeSet();
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// This function will list the current change set in the current client instance.
		/// </summary>
		protected void RefreshChangeSet()
		{
            ClientInstance.Refresh();

            ClientInstance.UpdateKnownChanges_All(true);

            ChangeSetItemColl csic = ClientInstance.InternalChangeSet_GetItems(true);
		}

		/// <summary>
		/// This small function simply formats and displays the data.
		/// </summary>
		/// <param name="fileName">Filename to display.</param>
		/// <param name="fileLength">Length of file to display.</param>
		/// <param name="fileVersion">Version of file to display.</param>
		protected void LogDetails( string fileName, string fileLength, string fileVersion )
		{
			VaultLog( fileName.PadRight( 16, ' ' ) + fileLength.PadRight( 16, ' ' ) + fileVersion );
		}


		#endregion
	}
}
