#region GPL License.
/* 
 * SourceGear Vault Integration Tasks for NAnt
 * Copyright (C) 2003  Brian M. Schkerke

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#endregion

using System;
using System.Collections;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;
using VaultClientNetLib;
using VaultClientOperationsLib;

namespace NAnt.Vault.Tasks
{
	/// <summary>
	/// This task lists all of the files in a specified folder and requires appropriate access
	/// permissions.
	/// </summary>
	/// <example>
	///   <code><![CDATA[
	///     <vaultundochangeset
	///			url="http://localhost"
	///			username="username"
	///			password="password"
	///			repository="repository"
	///			path="/Project/Directory" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultundochangeset")]
	public class VaultUndoChangeSet : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		private	string				pDirectoryPath			= "";
		#endregion
		
		#region Accessor Declarations
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string Repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}

		/// <summary>
		/// Gets or sets the directory path.
		/// </summary>
		/// <value></value>
		[TaskAttribute( "path", Required = true )]
		public string DirectoryPath
		{
			get	{ return pDirectoryPath;					}
			set	{ pDirectoryPath = value;					}
		}
		#endregion
		
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();

			try
			{
				SelectRepository( Repository );
				UndoChangeSet(DirectoryPath);
			}
			finally
			{
				Logout();
			}
		}
		#endregion
		
		#region Support functions.
		/// <summary>
		/// This function will list the current change set in the current client instance.
		/// </summary>
		protected void UndoChangeSet(string strPath)
		{
			strPath = AppendLastSlash(strPath);

			ClientInstance.UpdateKnownChanges_All(true);

			ChangeSetItemColl csic = ClientInstance.InternalChangeSet_GetItems(true);
			
			foreach (ChangeSetItem csi in csic)
			{
				string strDisplayRepositoryPath = csi.DisplayRepositoryPath;
				string strRepositoryPath = "";
				int nIndex = strDisplayRepositoryPath.LastIndexOf('$');
				if (nIndex >= 0)
				{
					if (strDisplayRepositoryPath.Length >= strPath.Length)
					{
						strRepositoryPath = strDisplayRepositoryPath.Substring(0, strPath.Length);
					}
				}

				if (strRepositoryPath == strPath)
				{
					ClientInstance.InternalChangeSet_Undo(csi);
				}
			}
		}

		private string AppendLastSlash(string strPath)
		{
			if (strPath.Length > 0)
			{
				if (strPath[strPath.Length - 1] != '/')
				{
					return strPath + "/";
				}
			}

			return strPath;
		}
		#endregion
	}
}
